package com.telerik.examples.examples.autocomplete;

import android.app.Activity;
import android.content.Context;
import android.graphics.Color;
import android.graphics.Point;
import android.graphics.drawable.Drawable;
import android.os.Build;
import android.os.Bundle;
import android.support.annotation.ColorRes;
import android.support.annotation.Nullable;
import android.support.v4.content.res.ResourcesCompat;
import android.support.v7.widget.Toolbar;
import android.view.Display;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.TextView;

import com.telerik.examples.R;
import com.telerik.examples.common.fragments.ExampleFragmentBase;
import com.telerik.examples.examples.dataform.OnEditEndListener;
import com.telerik.widget.autocomplete.AutoCompleteAdapter;
import com.telerik.widget.autocomplete.DisplayMode;
import com.telerik.widget.autocomplete.RadAutoCompleteTextView;
import com.telerik.widget.autocomplete.TokenModel;
import com.telerik.widget.autocomplete.TokenView;
import com.telerik.widget.dataform.engine.EntityProperty;
import com.telerik.widget.dataform.engine.EntityPropertyCore;
import com.telerik.widget.dataform.visualization.DataFormPlaceholderLayoutManager;
import com.telerik.widget.dataform.visualization.RadDataForm;
import com.telerik.widget.dataform.visualization.core.CommitMode;
import com.telerik.widget.dataform.visualization.editors.DataFormAutoCompleteEditor;
import com.telerik.widget.dataform.visualization.editors.DataFormDateEditor;
import com.telerik.widget.dataform.visualization.editors.DataFormNumberPickerEditor;
import com.telerik.widget.dataform.visualization.editors.DataFormRadAutoCompleteEditor;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.List;

public class AutoCompleteFragment extends ExampleFragmentBase{

    private RadDataForm dataForm;
    private JSONArray jsonObjects;
    private List<TokenModel> data;

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_autocomplete, container, false);
        rootView.setFocusableInTouchMode(true);
        rootView.setFocusable(true);
        rootView.setClickable(true);

        dataForm = (RadDataForm)rootView.findViewById(R.id.flight_data_form);

        if (rootView.getTag().equals("large")) {
            dataForm.setFillViewport(true);
            dataForm.setLayoutManager(new DataFormPlaceholderLayoutManager(this.getActivity(), R.layout.autocomplete_placeholder));
        }
        dataForm.setEntity(new FlightForm());

        String jsonData = this.getJSONFile(R.raw.airports);
        try {
            JSONObject jObj = new JSONObject(jsonData);
            jsonObjects = jObj.getJSONArray("airports");
        }
        catch(JSONException ex){
            ex.printStackTrace();
        }

        List<String> list = new ArrayList<>();

        this.data = new ArrayList<TokenModel>();
        for (int i = 0; i < jsonObjects.length(); i++){
            String fullName = "";
            String abbreviationName = "";

            try {
                fullName = (String)jsonObjects.getJSONObject(i).get("FIELD2");
                abbreviationName = (String)jsonObjects.getJSONObject(i).get("FIELD5");
            }
            catch( JSONException ex){
                ex.printStackTrace();
            }

            String text = fullName + ", " + abbreviationName;

            TokenModel current = new TokenModel(text, null);
            list.add(text);

            this.data.add(current);
        }

        ((EntityPropertyCore)this.dataForm.getPropertyByName("From")).updateValues(list.toArray());

        AutoCompleteAdapter adapter = new AutoCompleteAdapter(this.getContext(), this.data, R.layout.suggestion_item_layout);
        adapter.setHighlightColor(Color.parseColor("#187390"));

        Display display = this.getActivity().getWindowManager().getDefaultDisplay();
        Point size = new Point();
        display.getSize(size);
        int height = size.y;

        DataFormRadAutoCompleteEditor fromEditor = (DataFormRadAutoCompleteEditor) dataForm.getExistingEditorForProperty("From");
        TextView fromHeader = (TextView) fromEditor.getHeaderView();
        fromHeader.setTextColor(Color.parseColor("#187390"));
        fromEditor.getAutoCompleteView().setBackgroundResource(R.drawable.autocomplete_backgound);
        RadAutoCompleteTextView fromAutocomplete = (RadAutoCompleteTextView) fromEditor.getEditorView();

        fromEditor.updateAdapter();

        fromAutocomplete.setDisplayMode(DisplayMode.PLAIN);
        fromAutocomplete.setSuggestionViewHeight(height/4);

        DataFormRadAutoCompleteEditor toEditor = (DataFormRadAutoCompleteEditor) dataForm.getExistingEditorForProperty("To");
        TextView toHeader = (TextView) toEditor.getHeaderView();
        toHeader.setTextColor(Color.parseColor("#187390"));
        toEditor.getAutoCompleteView().setBackgroundResource(R.drawable.autocomplete_backgound);
        RadAutoCompleteTextView toAutocomplete = (RadAutoCompleteTextView) toEditor.getEditorView();
        toEditor.setDisplayMode(DisplayMode.PLAIN);
        toEditor.setAdapter(adapter);
        toAutocomplete.setSuggestionViewHeight(height/4 - 55);
        fromAutocomplete.setAutocompleteHint("Airport or city");
        toAutocomplete.setAutocompleteHint("Airport or city");

        DataFormDateEditor dateEditor = (DataFormDateEditor) dataForm.getExistingEditorForProperty("Date");
        TextView dateHeader = (TextView) dateEditor.getHeaderView();
        dateHeader.setTextColor(Color.parseColor("#187390"));

        DataFormNumberPickerEditor travelersEditor = (DataFormNumberPickerEditor) dataForm.getExistingEditorForProperty("Travelers");
        TextView travelersHeader = (TextView) travelersEditor.getHeaderView();
        travelersHeader.setTextColor(Color.parseColor("#187390"));

        if (!rootView.getTag().equals("large")) {
            Toolbar tb = (Toolbar) rootView.findViewById(R.id.toolbar_autocomplete);

            tb.setTitle("Search Flights");
            tb.setTitleTextColor(ResourcesCompat.getColor(getResources(), R.color.autocomplete_primary, null));
        }

        return rootView;
    }

    public String getJSONFile(int asset) {
        String json;
        try {
            InputStream is = getResources().openRawResource(asset);
            Writer writer = new StringWriter();
            char[] buffer = new char[1024];
            try {
                Reader reader = new BufferedReader(new InputStreamReader(is, "UTF-8"));
                int n;
                while ((n = reader.read(buffer)) != -1) {
                    writer.write(buffer, 0, n);
                }
            } finally {
                is.close();
            }

            json = writer.toString();
        } catch (IOException ex) {
            ex.printStackTrace();
            return null;
        }
        return json;
    }
}
