package com.telerik.examples.examples.autocomplete;

import android.graphics.Color;
import android.graphics.Point;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v7.widget.GridLayoutManager;
import android.view.Display;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.telerik.examples.R;
import com.telerik.examples.common.fragments.ExampleFragmentBase;
import com.telerik.widget.autocomplete.AutoCompleteAdapter;
import com.telerik.widget.autocomplete.CompletionMode;
import com.telerik.widget.autocomplete.DisplayMode;
import com.telerik.widget.autocomplete.LayoutMode;
import com.telerik.widget.autocomplete.RadAutoCompleteTextView;
import com.telerik.widget.autocomplete.TokenAddedListener;
import com.telerik.widget.autocomplete.TokenModel;
import com.telerik.widget.autocomplete.TokenRemovedListener;
import com.telerik.widget.list.ListViewAdapter;
import com.telerik.widget.list.RadListView;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.List;

public class AutoCompleteTokens extends ExampleFragmentBase {

    private RadAutoCompleteTextView autocomplete;
    private AutoCompleteAdapter adapter;
    private JSONArray jsonObjects;
    private List<TokenModel> data;

    private RadListView listView;
    private CategoryAdapter listViewAdapter;

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_tokens, container, false);
        rootView.setFocusableInTouchMode(true);
        rootView.setFocusable(true);
        rootView.setClickable(true);

        autocomplete = (RadAutoCompleteTextView) rootView.findViewById(R.id.autocomplete);

        String jsonData = this.getJSONFile(R.raw.categories);
        try {
            JSONObject jObj = new JSONObject(jsonData);
            jsonObjects = jObj.getJSONArray("data");
        }
        catch(JSONException ex){
            ex.printStackTrace();
        }

        this.data = new ArrayList<TokenModel>();
        for (int i = 0; i < jsonObjects.length(); i++){
            TokenModel current = null;
            try {
                current = new TokenModel((String)jsonObjects.getJSONObject(i).get("name"), null);
            } catch (JSONException e) {
                e.printStackTrace();
            }

            this.data.add(current);
        }

        this.adapter = new AutoCompleteAdapter(this.getContext(), this.data, R.layout.suggestion_item_layout);
        this.adapter.setCompletionMode(CompletionMode.CONTAINS);
        this.adapter.setHighlightColor(Color.parseColor("#2C95AC"));
        this.autocomplete.setAdapter(this.adapter);

        Display display = this.getActivity().getWindowManager().getDefaultDisplay();
        Point size = new Point();
        display.getSize(size);
        int height = size.y;
        int width = size.x;
        autocomplete.setSuggestionViewHeight(height/4);
        Drawable img = getResources().getDrawable(R.drawable.search_autocmp);
        autocomplete.setAutocompleteIcon(img);
        autocomplete.setTokensLayoutMode(LayoutMode.HORIZONTAL);

        List<Drawable> images = this.loadImages();
        this.listView = (RadListView) rootView.findViewById(R.id.tokens_list);

        int widthOfImage = width/3;
        int numberOfColumns = 3;
        if(rootView.getTag().equals("large")){
            widthOfImage = width/5;
            numberOfColumns = 5;
        }
        this.listViewAdapter = new CategoryAdapter(images, this.getContext(), R.layout.autocomplete_item_image, widthOfImage);
        this.listView.setAdapter(this.listViewAdapter);

        TokenAddedListener tokenAddedListener = new TokenAddedListener() {
            @Override
            public void onTokenAdded(RadAutoCompleteTextView autoComplete, TokenModel token) {
                listViewAdapter.showRandomNumberItems();
                listView.requestLayout();
            }
        };

        TokenRemovedListener tokenRemovedListener = new TokenRemovedListener() {
            @Override
            public void onTokenRemoved(RadAutoCompleteTextView autoComplete, TokenModel token) {
                listViewAdapter.showRandomNumberItems();
                listView.requestLayout();
            }
        };

        this.autocomplete.addTokenAddedListener(tokenAddedListener);
        this.autocomplete.addTokenRemovedListener(tokenRemovedListener);
        this.autocomplete.setBackgroundColor(Color.WHITE);
        this.autocomplete.setDisplayMode(DisplayMode.TOKENS);

        GridLayoutManager gridLayout = new GridLayoutManager(getActivity(), numberOfColumns);
        this.listView.setLayoutManager(gridLayout);

        return rootView;
    }

    private List<Drawable> loadImages() {
        List<Drawable> images = new ArrayList<Drawable>();
        String imageNameCommon = "content";

        for (int i = 1; i < 32; i++){
            String currentName = imageNameCommon + i;

            int drInt = getResources().getIdentifier(currentName, "drawable", "com.telerik.examples");
            Drawable dr = getResources().getDrawable(drInt);
            images.add(dr);
        }

        return images;
    }

    public String getJSONFile(int asset) {
        String json;
        try {
            InputStream is = getResources().openRawResource(asset);
            Writer writer = new StringWriter();
            char[] buffer = new char[1024];
            try {
                Reader reader = new BufferedReader(new InputStreamReader(is, "UTF-8"));
                int n;
                while ((n = reader.read(buffer)) != -1) {
                    writer.write(buffer, 0, n);
                }
            } finally {
                is.close();
            }

            json = writer.toString();
        } catch (IOException ex) {
            ex.printStackTrace();
            return null;
        }
        return json;
    }
}
