package com.telerik.examples.examples.gauges;

import android.annotation.TargetApi;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.SweepGradient;
import android.graphics.drawable.GradientDrawable;
import android.os.Build;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewTreeObserver;
import android.widget.Button;
import com.telerik.android.common.Util;
import com.telerik.examples.R;
import com.telerik.examples.common.DataClass;
import com.telerik.examples.common.fragments.ExampleFragmentBase;
import com.telerik.widget.chart.engine.databinding.FieldNameDataPointBinding;
import com.telerik.widget.chart.visualization.cartesianChart.CartesianChartGrid;
import com.telerik.widget.chart.visualization.cartesianChart.GridLineRenderMode;
import com.telerik.widget.chart.visualization.cartesianChart.RadCartesianChartView;
import com.telerik.widget.chart.visualization.cartesianChart.axes.CategoricalAxis;
import com.telerik.widget.chart.visualization.cartesianChart.axes.LinearAxis;
import com.telerik.widget.chart.visualization.cartesianChart.series.categorical.LineSeries;
import com.telerik.widget.gauge.RadRadialGaugeView;
import com.telerik.widget.indicators.GaugeBarIndicatorCapMode;
import com.telerik.widget.indicators.GaugeIndicator;
import com.telerik.widget.indicators.GaugeRadialBarIndicator;
import com.telerik.widget.scales.GaugeRadialScale;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Random;

import static android.content.res.Configuration.ORIENTATION_LANDSCAPE;


public class RadialGaugeExample extends ExampleFragmentBase {

    RadCartesianChartView chart;
    RadRadialGaugeView gauge;
    GaugeRadialScale scale;
    int[] transparentColors;
    int[] colors;
    float[] indicatorValues;
    String[] days;
    ArrayList<DataClass> moveData;
    ArrayList<DataClass> exerciseData;
    ArrayList<DataClass> standData;
    LineSeries moveSeries;
    LineSeries exerciseSeries;
    LineSeries standSeries;
    GradientDrawable moveGradient;
    GradientDrawable exerciseGradient;
    GradientDrawable standGradient;
    int sdk;

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.radial_gauge_example, container, false);
        gauge = Util.getLayoutPart(rootView, R.id.radial_gauge, RadRadialGaugeView.class);
        chart = Util.getLayoutPart(rootView, R.id.cartesian_chart, RadCartesianChartView.class);
        transparentColors = new int[] {
                Color.argb(51,245,142,0),
                Color.argb(51,97,186,0),
                Color.argb(51,0,179,230)
        };

        colors = new int[] {
                Color.argb(255,255,192,0),
                Color.argb(255,245,142,0),
                Color.argb(255,197,255,0),
                Color.argb(255,97,186,0),
                Color.argb(255,0,255,255),
                Color.argb(255,0,179,230),
        };

        sdk = android.os.Build.VERSION.SDK_INT;
        moveGradient = new GradientDrawable(GradientDrawable.Orientation.TOP_BOTTOM, new int[] {colors[5], colors[4]});
        exerciseGradient = new GradientDrawable(GradientDrawable.Orientation.TOP_BOTTOM, new int[] {colors[3], colors[2]});
        standGradient = new GradientDrawable(GradientDrawable.Orientation.TOP_BOTTOM, new int[] {colors[1], colors[0]});
        indicatorValues = new float[] { 87, 67, 81 };
        days = new String[] { "Sat", "Sun", "Mon", "Tue", "Wed", "Thu", "Today" };
        moveData = new ArrayList<>();
        exerciseData = new ArrayList<>();
        standData = new ArrayList<>();
        prepareChartData();
        setupChart();
        setupGauge();

        Button moveBtn = Util.getLayoutPart(rootView, R.id.radial_gauge_example_move_btn, Button.class);
        moveBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                moveButtonClicked();
            }
        });

        Button exerciseBtn = Util.getLayoutPart(rootView, R.id.radial_gauge_example_exercise_btn, Button.class);
        exerciseBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                exerciseButtonClicked();
            }
        });

        Button standBtn = Util.getLayoutPart(rootView, R.id.radial_gauge_example_stand_btn, Button.class);
        standBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                standButtonClicked();
            }
        });

        if (getResources().getConfiguration().orientation == ORIENTATION_LANDSCAPE) {

        }

        return rootView;
    }

    private void styleGradientIndicators() {
        for (int i = 1; i < 6; i+=2) {
            GaugeIndicator indicator = scale.getIndicators().get(i);
            float centerX = (gauge.getWidth()/2);
            float centerY = (gauge.getHeight()/2);
            int[] gradientColors = new int[] { colors[i], colors[i-1] };
            SweepGradient gradient = new SweepGradient(centerX ,centerY,gradientColors,new float[] {0, 1});
            final Matrix matrix = new Matrix();
            matrix.setRotate(-105, centerX, centerY);
            gradient.setLocalMatrix(matrix);
            indicator.getFillPaint().setShader(gradient);
        }
    }

    private void setupGauge() {
        scale = new GaugeRadialScale(getContext());
        scale.setStartAngle(-90);
        scale.setSweepAngle(360);
        scale.setLineVisible(false);
        scale.setLabelsVisible(false);
        scale.setTicksVisible(false);

        for (int i = 0; i < 3; i++) {
            GaugeRadialBarIndicator trnspIndicator = new GaugeRadialBarIndicator(getActivity());
            trnspIndicator.setMinimum(0);
            trnspIndicator.setMaximum(100);
            trnspIndicator.setFillColor(transparentColors[i]);
            trnspIndicator.setBarWidth(0.2f);
            trnspIndicator.setLocation(0.5f +  i * 0.25f);
            scale.addIndicator(trnspIndicator);

            GaugeRadialBarIndicator gradientIndicator = new GaugeRadialBarIndicator(getActivity());
            gradientIndicator.setRange(0, indicatorValues[i]);
            gradientIndicator.setCap(GaugeBarIndicatorCapMode.ROUND);
            gradientIndicator.setBarWidth(0.2f);
            gradientIndicator.setLocation(0.5f +  i * 0.25f);
            gradientIndicator.setAnimated(true);
            gradientIndicator.setAnimationDuration(500);
            scale.addIndicator(gradientIndicator);
        }

        gauge.addScale(scale);
        gauge.getViewTreeObserver().addOnGlobalLayoutListener(new ViewTreeObserver.OnGlobalLayoutListener() {
            @TargetApi(Build.VERSION_CODES.JELLY_BEAN)
            @Override
            public void onGlobalLayout() {
                styleGradientIndicators();
                gauge.getViewTreeObserver().removeOnGlobalLayoutListener(this);
            }
        });
    }

    @TargetApi(Build.VERSION_CODES.JELLY_BEAN)
    private void setupChart() {
        moveSeries = new LineSeries();
        moveSeries.setCategoryBinding(new FieldNameDataPointBinding("category"));
        moveSeries.setValueBinding(new FieldNameDataPointBinding("value"));
        moveSeries.setData(moveData);
        moveSeries.setStrokeColor(Color.WHITE);

        exerciseSeries = new LineSeries();
        exerciseSeries.setCategoryBinding(new FieldNameDataPointBinding("category"));
        exerciseSeries.setValueBinding(new FieldNameDataPointBinding("value"));
        exerciseSeries.setData(exerciseData);
        exerciseSeries.setStrokeColor(Color.WHITE);

        standSeries = new LineSeries();
        standSeries.setCategoryBinding(new FieldNameDataPointBinding("category"));
        standSeries.setValueBinding(new FieldNameDataPointBinding("value"));
        standSeries.setData(standData);
        standSeries.setStrokeColor(Color.WHITE);

        LinearAxis vAxis = new LinearAxis();
        vAxis.setShowLabels(false);
        vAxis.setMinimum(0);
        vAxis.setMaximum(8);
        vAxis.setMajorStep(1.6);
        vAxis.setShowLine(false);

        CategoricalAxis hAxis = new CategoricalAxis();
        chart.setVerticalAxis(vAxis);
        chart.setHorizontalAxis(hAxis);
        hAxis.setLineColor(Color.WHITE);

        CartesianChartGrid grid = new CartesianChartGrid();
        chart.setGrid(grid);
        grid.setMajorYLinesRenderMode(GridLineRenderMode.INNER);
        grid.setLineThickness(1);
        grid.setLineColor(Color.WHITE);
        moveButtonClicked();
    }

    @TargetApi(Build.VERSION_CODES.JELLY_BEAN)
    private void moveButtonClicked() {
        chart.beginUpdate();
        if (chart.getSeries().size() > 0) {
            chart.getSeries().remove(0);
        }

        chart.getSeries().add(moveSeries);
        if(sdk < android.os.Build.VERSION_CODES.JELLY_BEAN) {
            chart.setBackgroundDrawable(moveGradient);
        } else {
            chart.setBackground(moveGradient);
        }
        chart.endUpdate();
    }

    @TargetApi(Build.VERSION_CODES.JELLY_BEAN)
    private void exerciseButtonClicked() {
        chart.beginUpdate();
        if (chart.getSeries().size() > 0) {
            chart.getSeries().remove(0);
        }

        chart.getSeries().add(exerciseSeries);
        if(sdk < android.os.Build.VERSION_CODES.JELLY_BEAN) {
            chart.setBackgroundDrawable(exerciseGradient);
        } else {
            chart.setBackground(exerciseGradient);
        }
        chart.endUpdate();
    }

    @TargetApi(Build.VERSION_CODES.JELLY_BEAN)
    private void standButtonClicked() {
        chart.beginUpdate();
        if (chart.getSeries().size() > 0) {
            chart.getSeries().remove(0);
        }

        chart.getSeries().add(standSeries);
        if(sdk < android.os.Build.VERSION_CODES.JELLY_BEAN) {
            chart.setBackgroundDrawable(standGradient);
        } else {
            chart.setBackground(standGradient);
        }
        chart.endUpdate();
    }

    private void prepareChartData() {
        Random r = new Random();
        for (int i = 6; i >= 0; i--) {
            Calendar d = Calendar.getInstance();
            d.add(Calendar.DATE, i * -1);
            int value = r.nextInt(8);
            moveData.add(new DataClass(days[6 - i], value));

            value = r.nextInt(8);
            exerciseData.add(new DataClass(days[6 - i], value));

            value = r.nextInt(8);
            standData.add(new DataClass(days[6 - i], value));
        }
    }

    @Override
    public void onStart() {
        super.onStart();
        gauge.animateGauge();
    }
}
