package com.telerik.examples.examples.gauges;

import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.support.annotation.Nullable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.telerik.android.common.Util;
import com.telerik.examples.R;
import com.telerik.examples.common.fragments.ExampleFragmentBase;
import com.telerik.widget.gauge.RadRadialGaugeView;
import com.telerik.widget.indicators.GaugeRadialBarIndicator;
import com.telerik.widget.indicators.GaugeRadialNeedle;
import com.telerik.widget.scales.GaugeRadialScale;
import com.telerik.widget.scales.GaugeScaleLabelsLayoutMode;
import com.telerik.widget.scales.GaugeScaleTicksLayoutMode;

import java.util.Random;

public class RadialNeedleExample extends ExampleFragmentBase {
    RadRadialGaugeView topLeftGauge;
    RadRadialGaugeView topRightGauge;
    RadRadialGaugeView bottomLeftGauge;
    RadRadialGaugeView bottomRightGauge;
    RadRadialGaugeView lastGauge;

    GaugeRadialNeedle animatedNeedle1;
    GaugeRadialNeedle animatedNeedle2;
    GaugeRadialNeedle animatedNeedle3;
    GaugeRadialNeedle animatedNeedle4;
    GaugeRadialNeedle animatedNeedle5;

    Handler timerHandler = new Handler();
    Runnable timerRunnable = new Runnable() {
        @Override
        public void run() {
            setGaugeValues();
            timerHandler.postDelayed(this, 2000);
        }
    };

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.radial_gauge_needle_gallery, container, false);
        topLeftGauge = Util.getLayoutPart(rootView, R.id.top_left_gauge, RadRadialGaugeView.class);
        topRightGauge = Util.getLayoutPart(rootView, R.id.top_right_gauge, RadRadialGaugeView.class);
        bottomLeftGauge = Util.getLayoutPart(rootView, R.id.bottom_left_gauge, RadRadialGaugeView.class);
        bottomRightGauge = Util.getLayoutPart(rootView, R.id.bottom_right_gauge, RadRadialGaugeView.class);
        lastGauge = Util.getLayoutPart(rootView, R.id.last_gauge, RadRadialGaugeView.class);
        setupTopLeftGauge();
        setupTopRightGauge();
        setupBottomLeftGauge();
        setupBottomRightGauge();
        setupLastGauge();
        return rootView;
    }

    @Override
    public void onStart() {
        super.onStart();
        timerHandler.postDelayed(timerRunnable, 0);
    }

    @Override
    public void onPause() {
        super.onPause();
        timerHandler.removeCallbacks(timerRunnable);
    }

    private void setupTopLeftGauge() {
        GaugeRadialScale scale = new GaugeRadialScale(getActivity());
        scale.setRadius(0.85f);
        scale.setLabelsVisible(false);
        scale.setMinorTicksWidth(8);
        scale.setMajorTicksWidth(24);
        scale.setMajorTicksHeight(3);
        scale.setMajorTicksCount(20);
        scale.setMinorTicksCount(2);
        scale.setTicksOffset(8);
        scale.setMajorTicksStrokeColor(Color.GRAY);
        scale.setStrokeColor(Color.GRAY);
        scale.setStrokeWidth(6);
        scale.setSweepAngle(360);
        topLeftGauge.addScale(scale);

        animatedNeedle1 = new GaugeRadialNeedle(getActivity());
        animatedNeedle1.setAnimated(true);
        animatedNeedle1.setValue(80);
        animatedNeedle1.setBottomWidth(getResources().getDimension(R.dimen.gauge_needle_thin));
        animatedNeedle1.setTopWidth(getResources().getDimension(R.dimen.gauge_needle_thin));
        animatedNeedle1.setCircleRadius(getResources().getDimension(R.dimen.gauge_needle_radius));
        animatedNeedle1.setLength(0.7f);
        animatedNeedle1.setFillColor(Color.rgb(110, 184, 227));
        animatedNeedle1.setCircleFillColor(Color.rgb(110, 184, 227));
        scale.addIndicator(animatedNeedle1);
    }

    private  void setupTopRightGauge() {
        GaugeRadialScale scale = new GaugeRadialScale(getActivity());
        scale.setRadius(0.75f);
        scale.setStartAngle(135);
        scale.setSweepAngle(270);
        scale.setLineVisible(false);
        scale.setMajorTicksCount(19);
        scale.setMajorTicksHeight(3);
        scale.setMajorTicksWidth(12);
        scale.setMinorTicksCount(0);
        scale.setRange(0, 180);
        scale.setLabelsCount(10);
        scale.getLabelsPaint().setTextSize(getResources().getDimension(R.dimen.gauge_scale_label_size));
        scale.setLabelsOffset(getResources().getDimension(R.dimen.gauge_scale_label_offset));
        topRightGauge.addScale(scale);

        animatedNeedle2 = new GaugeRadialNeedle(getActivity());
        animatedNeedle2.setLength(0.65f);
        animatedNeedle2.setAnimated(true);
        animatedNeedle2.setTopWidth(1);
        animatedNeedle2.setBottomWidth(getResources().getDimension(R.dimen.gauge_needle_thin));
        animatedNeedle2.setCircleRadius(getResources().getDimension(R.dimen.gauge_needle_radius));
        animatedNeedle2.setCircleFillColor(Color.RED);
        animatedNeedle2.setFillColor(Color.RED);
        animatedNeedle2.setValue(140);
        scale.addIndicator(animatedNeedle2);

        GaugeRadialBarIndicator greenBar = new GaugeRadialBarIndicator(getActivity());
        greenBar.setRange(0, 60);
        greenBar.setLocation(0.85f);
        greenBar.setFillColor(Color.rgb(163, 213, 84));
        scale.addIndicator(greenBar);

        GaugeRadialBarIndicator orangeBar = new GaugeRadialBarIndicator(getActivity());
        orangeBar.setRange(61, 120);
        orangeBar.setLocation(0.85f);
        orangeBar.setFillColor(Color.rgb(222, 158, 38));
        scale.addIndicator(orangeBar);

        GaugeRadialBarIndicator redBar = new GaugeRadialBarIndicator(getActivity());
        redBar.setRange(121, 180);
        redBar.setLocation(0.85f);
        redBar.setFillColor(Color.RED);
        scale.addIndicator(redBar);
    }

    private void setupBottomLeftGauge() {
        GaugeRadialScale scale = new GaugeRadialScale(getActivity());
        scale.setRange(0, 180);
        scale.setRadius(0.85f);
        scale.setStartAngle(135);
        scale.setSweepAngle(270);
        scale.setStrokeColor(Color.GRAY);
        scale.setStrokeWidth(6);

        scale.setMinorTicksWidth(8);
        scale.setMajorTicksWidth(24);
        scale.setMajorTicksHeight(3);
        scale.setMajorTicksCount(19);
        scale.setMinorTicksCount(2);
        scale.setTicksOffset(8);
        scale.setMajorTicksStrokeColor(Color.GRAY);

        scale.setLabelsCount(10);
        scale.getLabelsPaint().setTextSize(getResources().getDimension(R.dimen.gauge_scale_label_size));
        scale.setLabelsOffset(getResources().getDimension(R.dimen.gauge_scale_label_offset));
        scale.setLabelsOffset(scale.getLabelsOffset() + 10);
        bottomLeftGauge.addScale(scale);

        GaugeRadialNeedle blueNeedle = new GaugeRadialNeedle(getActivity());
        blueNeedle.setLength(0.9f);
        blueNeedle.setFillColor(Color.rgb(110, 184, 227));
        blueNeedle.setCircleRadius(0);
        blueNeedle.setValue(160);
        blueNeedle.setTopWidth(1);
        blueNeedle.setBottomWidth(getResources().getDimension(R.dimen.gauge_needle_thin));
        blueNeedle.setOffset(getResources().getDimension(R.dimen.gauge_needle_offset));
        scale.addIndicator(blueNeedle);

        animatedNeedle3 = new GaugeRadialNeedle(getActivity());
        animatedNeedle3.setAnimated(true);
        animatedNeedle3.setLength(0.9f);
        animatedNeedle3.setFillColor(Color.RED);
        animatedNeedle3.setCircleRadius(getResources().getDimension(R.dimen.gauge_needle_radius));
        animatedNeedle3.setCircleFillColor(Color.RED);
        animatedNeedle3.setTopWidth(1);
        animatedNeedle3.setBottomWidth(getResources().getDimension(R.dimen.gauge_needle_thin));
        animatedNeedle3.setOffset(getResources().getDimension(R.dimen.gauge_needle_offset));
        animatedNeedle3.setValue(100);
        scale.addIndicator(animatedNeedle3);
    }

    private void setupBottomRightGauge() {
        GaugeRadialScale scale = new GaugeRadialScale(getActivity());
        scale.setRadius(0.85f);
        scale.setStartAngle(180);
        scale.setSweepAngle(180);
        scale.setStrokeColor(Color.GRAY);
        scale.setStrokeWidth(6);

        scale.setMajorTicksCount(13);
        scale.setMinorTicksCount(2);
        scale.setMajorTicksWidth(28);
        scale.setMinorTicksWidth(10);
        scale.setMajorTicksHeight(3);

        scale.setLabelsVisible(false);
        bottomRightGauge.addScale(scale);

        animatedNeedle4 = new GaugeRadialNeedle(getActivity());
        animatedNeedle4.setLength(0.75f);
        animatedNeedle4.setAnimated(true);
        animatedNeedle4.setFillColor(Color.rgb(110, 184, 227));
        animatedNeedle4.setCircleFillColor(Color.rgb(110, 184, 227));
        animatedNeedle4.setCircleRadius(getResources().getDimension(R.dimen.gauge_needle_radius));
        animatedNeedle4.setBottomWidth(getResources().getDimension(R.dimen.gauge_needle_thick));
        animatedNeedle4.setValue(40);
        scale.addIndicator(animatedNeedle4);

    }

    private void setupLastGauge() {
        GaugeRadialScale scale = new GaugeRadialScale(getActivity());
        scale.setStartAngle(270);
        scale.setSweepAngle(90);
        scale.setRange(50, 150);
        scale.setRadius(0.70f);
        scale.setStrokeColor(Color.GRAY);
        scale.setStrokeWidth(6);
        scale.setTicksLayoutMode(GaugeScaleTicksLayoutMode.OUTER);
        scale.setMajorTicksCount(7);
        scale.setMinorTicksCount(0);
        scale.setMajorTicksWidth(20);
        scale.setTicksOffset(10);
        scale.setLabelsCount(2);
        scale.setLabelsLayoutMode(GaugeScaleLabelsLayoutMode.OUTER);
        scale.getLabelsPaint().setTextSize(getResources().getDimension(R.dimen.gauge_scale_label_size));
        scale.setLabelsOffset(getResources().getDimension(R.dimen.gauge_scale_label_offset));
        lastGauge.addScale(scale);

        animatedNeedle5 = new GaugeRadialNeedle(getActivity());
        animatedNeedle5.setAnimated(true);
        animatedNeedle5.setFillColor(Color.RED);
        animatedNeedle5.setCircleFillColor(Color.RED);
        animatedNeedle5.setLength(0.9f);
        animatedNeedle5.setValue(110);
        animatedNeedle5.setCircleRadius(getResources().getDimension(R.dimen.gauge_needle_radius));
        animatedNeedle5.setTopWidth(getResources().getDimension(R.dimen.gauge_needle_thin));
        animatedNeedle5.setBottomWidth(getResources().getDimension(R.dimen.gauge_needle_thin));
        scale.addIndicator(animatedNeedle5);
    }

    private void setGaugeValues() {
        Random r = new Random();
        float value1 = r.nextInt(100);
        float value2 = r.nextInt(180);
        float value3 = r.nextInt(180);
        float value4 = r.nextInt(100);
        float value5 = r.nextInt(100) + 50;

        animatedNeedle1.setValue(value1);
        animatedNeedle2.setValue(value2);
        animatedNeedle3.setValue(value3);
        animatedNeedle4.setValue(value4);
        animatedNeedle5.setValue(value5);
    }
}
