package com.telerik.examples.examples.gauges;

import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.support.annotation.Nullable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.telerik.android.common.Util;
import com.telerik.examples.R;
import com.telerik.examples.common.fragments.ExampleFragmentBase;
import com.telerik.widget.gauge.RadRadialGaugeView;
import com.telerik.widget.indicators.GaugeRadialBarIndicator;
import com.telerik.widget.indicators.GaugeRadialNeedle;
import com.telerik.widget.scales.GaugeRadialScale;
import com.telerik.widget.scales.GaugeScaleLabelsLayoutMode;
import com.telerik.widget.scales.GaugeScaleTicksLayoutMode;

import java.util.Random;

public class RadialVariousIndicatorsExample extends ExampleFragmentBase {
    RadRadialGaugeView topLeftGauge;
    RadRadialGaugeView topRightGauge;
    RadRadialGaugeView bottomGauge;

    GaugeRadialBarIndicator topLeftGaugeBlueBar;
    GaugeRadialBarIndicator topLeftGaugeGreenBar;
    GaugeRadialNeedle topLeftGaugeNeedle;
    GaugeRadialBarIndicator topRightGaugeBlueBar;
    GaugeRadialNeedle topRightGaugeNeedle;
    GaugeRadialBarIndicator bottomGaugeGreenBar;
    GaugeRadialNeedle bottomGaugeNeedle;

    Handler timerHandler = new Handler();
    Runnable timerRunnable = new Runnable() {
        @Override
        public void run() {
            setGaugeValues();
            timerHandler.postDelayed(this, 2000);
        }
    };

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.radial_gauge_various_indicators_gallery, container, false);
        topLeftGauge = Util.getLayoutPart(rootView, R.id.top_left_gauge, RadRadialGaugeView.class);
        topRightGauge = Util.getLayoutPart(rootView, R.id.top_right_gauge, RadRadialGaugeView.class);
        bottomGauge = Util.getLayoutPart(rootView, R.id.bottom_gauge, RadRadialGaugeView.class);
        setupTopLeftGauge();
        setupTopRightGauge();
        setupBottomGauge();
        return rootView;
    }

    @Override
    public void onStart() {
        super.onStart();
        timerHandler.postDelayed(timerRunnable, 0);
    }

    @Override
    public void onPause() {
        super.onPause();
        timerHandler.removeCallbacks(timerRunnable);
    }

    private void setupTopLeftGauge() {
        GaugeRadialScale scale = new GaugeRadialScale(getActivity());
        scale.setStartAngle(180);
        scale.setSweepAngle(360);
        scale.setLineVisible(false);
        scale.setRadius(0.95f);

        scale.setMajorTicksCount(24);
        scale.setMinorTicksCount(2);
        scale.setMajorTicksFillColor(Color.GRAY);
        scale.setMinorTicksFillColor(Color.GRAY);
        scale.setTicksLayoutMode(GaugeScaleTicksLayoutMode.OUTER);
        scale.setMajorTicksWidth(16);
        scale.setMajorTicksHeight(3);
        scale.setMinorTicksWidth(7);
        scale.setMinorTicksHeight(3);

        scale.setLabelsVisible(false);
        topLeftGauge.addScale(scale);

        GaugeRadialBarIndicator grayBar = new GaugeRadialBarIndicator(getActivity());
        grayBar.setBarWidth(0.1f);
        grayBar.setLocation(0.94f);
        grayBar.setRange(0, 100);
        grayBar.setFillColor(Color.rgb(221, 221, 221));
        scale.addIndicator(grayBar);

        topLeftGaugeBlueBar = new GaugeRadialBarIndicator(getActivity());
        topLeftGaugeBlueBar.setAnimated(true);
        topLeftGaugeBlueBar.setBarWidth(0.1f);
        topLeftGaugeBlueBar.setLocation(0.94f);
        topLeftGaugeBlueBar.setRange(0, 35);
        topLeftGaugeBlueBar.setFillColor(Color.rgb(33, 197, 254));
        scale.addIndicator(topLeftGaugeBlueBar);

        topLeftGaugeGreenBar = new GaugeRadialBarIndicator(getActivity());
        topLeftGaugeGreenBar.setAnimated(true);
        topLeftGaugeGreenBar.setLocation(0.80f);
        topLeftGaugeGreenBar.setRange(0, 40);
        topLeftGaugeGreenBar.setFillColor(Color.rgb(163, 213, 84));
        scale.addIndicator(topLeftGaugeGreenBar);

        topLeftGaugeNeedle = new GaugeRadialNeedle(getActivity());
        topLeftGaugeNeedle.setAnimated(true);
        topLeftGaugeNeedle.setLength(0.80f);
        topLeftGaugeNeedle.setValue(40);
        topLeftGaugeNeedle.setBottomWidth(getResources().getDimension(R.dimen.gauge_needle_thin));
        topLeftGaugeNeedle.setTopWidth(getResources().getDimension(R.dimen.gauge_needle_thin));
        topLeftGaugeNeedle.setCircleRadius(getResources().getDimension(R.dimen.gauge_needle_radius));
        topLeftGaugeNeedle.setFillColor(Color.BLACK);
        topLeftGaugeNeedle.setCircleFillColor(Color.BLACK);
        scale.addIndicator(topLeftGaugeNeedle);
    }

    private void setupTopRightGauge() {
        GaugeRadialScale scale = new GaugeRadialScale(getActivity());
        scale.setStartAngle(225);
        scale.setSweepAngle(90);
        scale.setTicksVisible(false);
        scale.setLineVisible(false);
        scale.setLabelsCount(2);
        scale.setLabelsLayoutMode(GaugeScaleLabelsLayoutMode.OUTER);
        scale.setRange(0, 50);
        scale.setRadius(0.85f);
        scale.getLabelsPaint().setTextSize(getResources().getDimension(R.dimen.gauge_scale_label_size));
        scale.setLabelsOffset(getResources().getDimension(R.dimen.gauge_scale_label_offset));
        topRightGauge.addScale(scale);

        GaugeRadialBarIndicator grayBar = new GaugeRadialBarIndicator(getActivity());
        grayBar.setRange(0, 50);
        grayBar.setBarWidth(0.3f);
        grayBar.setFillColor(Color.rgb(221, 221, 221));
        grayBar.setLocation(0.85f);
        scale.addIndicator(grayBar);

        topRightGaugeBlueBar = new GaugeRadialBarIndicator(getActivity());
        topRightGaugeBlueBar.setAnimated(true);
        topRightGaugeBlueBar.setRange(0, 33.33f);
        topRightGaugeBlueBar.setBarWidth(0.3f);
        topRightGaugeBlueBar.setFillColor(Color.rgb(33, 197, 254));
        topRightGaugeBlueBar.setLocation(0.85f);
        scale.addIndicator(topRightGaugeBlueBar);

        topRightGaugeNeedle = new GaugeRadialNeedle(getActivity());
        topRightGaugeNeedle.setAnimated(true);
        topRightGaugeNeedle.setLength(0.95f);
        topRightGaugeNeedle.setValue(33.33f);
        topRightGaugeNeedle.setBottomWidth(getResources().getDimension(R.dimen.gauge_needle_thin));
        topRightGaugeNeedle.setTopWidth(getResources().getDimension(R.dimen.gauge_needle_thin));
        topRightGaugeNeedle.setCircleRadius(getResources().getDimension(R.dimen.gauge_needle_radius));
        topRightGaugeNeedle.setFillColor(Color.RED);
        topRightGaugeNeedle.setCircleFillColor(Color.RED);
        scale.addIndicator(topRightGaugeNeedle);

        topRightGauge.getTitle().setText("33.33");
        topRightGauge.getTitle().setTextSize(22);
        topRightGauge.setTitleVerticalOffset(70);
        topRightGauge.getTitle().setTextColor(Color.BLACK);
    }

    private void setupBottomGauge() {
        GaugeRadialScale scale = new GaugeRadialScale(getActivity());
        scale.setStartAngle(180);
        scale.setSweepAngle(180);
        scale.setTicksVisible(false);
        scale.setLineVisible(false);
        scale.setLabelsVisible(false);
        bottomGauge.addScale(scale);

        GaugeRadialBarIndicator grayBar = new GaugeRadialBarIndicator(getActivity());
        grayBar.setRange(0, 100);
        grayBar.setBarWidth(0.5f);
        grayBar.setFillColor(Color.rgb(221, 221, 221));
        scale.addIndicator(grayBar);

        bottomGaugeGreenBar = new GaugeRadialBarIndicator(getActivity());
        bottomGaugeGreenBar.setAnimated(true);
        bottomGaugeGreenBar.setRange(0, 75);
        bottomGaugeGreenBar.setBarWidth(0.5f);
        bottomGaugeGreenBar.setFillColor(Color.rgb(163, 213, 84));
        scale.addIndicator(bottomGaugeGreenBar);

        bottomGaugeNeedle = new GaugeRadialNeedle(getActivity());
        bottomGaugeNeedle.setAnimated(true);
        bottomGaugeNeedle.setLength(1);
        bottomGaugeNeedle.setFillColor(Color.BLACK);
        bottomGaugeNeedle.setCircleFillColor(Color.BLACK);
        bottomGaugeNeedle.setCircleRadius(getResources().getDimension(R.dimen.gauge_needle_radius));
        bottomGaugeNeedle.setBottomWidth(getResources().getDimension(R.dimen.gauge_needle_thick));
        bottomGaugeNeedle.setValue(75);
        scale.addIndicator(bottomGaugeNeedle);
    }

    private void setGaugeValues() {
        Random r = new Random();
        topLeftGaugeBlueBar.setMaximum(r.nextInt(100));

        topLeftGaugeGreenBar.setMaximum(r.nextInt(100));
        topLeftGaugeNeedle.setValue(topLeftGaugeGreenBar.getMaximum());

        topRightGaugeBlueBar.setMaximum(r.nextInt(50));
        topRightGaugeNeedle.setValue(topRightGaugeBlueBar.getMaximum());
        topRightGauge.getTitle().setText(String.format(getResources().getConfiguration().locale,"%.0f", topRightGaugeBlueBar.getMaximum()));

        bottomGaugeGreenBar.setMaximum(r.nextInt(100));
        bottomGaugeNeedle.setValue(bottomGaugeGreenBar.getMaximum());
    }
}
